@@ -28,9 +28,8 @@ class AppiumServer(Logger):
         assert self.service.is_running, 'Appium is not running'
         if version.startswith("1."):
             return f'http://127.0.0.1:{self.port}/wd/hub'
-        elif version.startswith("2."):
-            return f'http://127.0.0.1:{self.port}'
-        raise AssertionError('appium version not supported')
+        else:
+            return f'http://127.0.0.1:{self.port}'  # Appium >= 2.0
 
     def start(self):
         assert not self.service.is_running, 'Appium already running'
