@@ -9,17 +9,19 @@ from stf_client.exceptions import ForbiddenException
 
 from stf_appium_client.Logger import Logger
 from stf_appium_client.exceptions import DeviceNotFound, NotConnectedError
-from stf_appium_client.tools import lock
 from stf_client.api_client import ApiClient, Configuration
 from stf_client.api.user_api import UserApi
 from stf_client.api.devices_api import DevicesApi
 
+STATUS_ONLINE = 3
+
+
 class StfClient(Logger):
     DEFAULT_ALLOCATION_TIMEOUT_SECONDS = 900
 
     def __init__(self, host: str):
         """
-        OpenSTF Client consructor
+        STF Client constructor
         :param host: Server address of OpenSTF
         """
         super().__init__()
@@ -78,8 +80,7 @@ class StfClient(Logger):
         timeout = timeout_seconds * 1000
 
         api_instance = UserApi(self._client)
-        with lock():
-            api_response = api_instance.add_user_device_v2(serial, timeout=timeout)
+        api_response = api_instance.add_user_device_v2(serial, timeout=timeout)
         assert api_response.success, 'allocation fails'
         self.logger.info(f'{serial}: Allocated (timeout: {timeout_seconds})')
         device['owner'] = "me"
@@ -141,17 +142,17 @@ class StfClient(Logger):
         self.logger.debug(f'{serial}: releasing..')
 
         api_instance = UserApi(self._client)
-        with lock():
-            api_response = api_instance.delete_user_device_by_serial(serial)
+        api_response = api_instance.delete_user_device_by_serial(serial)
         assert api_response.success, 'release fails'
         device['owner'] = None
         self.logger.info(f'{serial}: released')
 
-    def list_devices(self, requirements: dict, fields: str = "") -> list:
+    def list_devices(self, requirements: dict, fields: str = "", available_filter: bool = True) -> list:
         """
         Get list of devices filtered by given requirements and optional extra fields
         :param requirements: filter dictionary
         :param fields: extra fields to include
+        :param available_filter: filter only available devices
         :return: list of objects that represent devices
         """
         req_keys = list(requirements.keys())
@@ -164,15 +165,15 @@ class StfClient(Logger):
         fields = uniq(req_keys)
 
         predicate = requirements.copy()
-
-        predicate.update(
-            dict(
-                present=True,
-                ready=True,
-                using=False,
-                owner=None,
-                status=3)  # 3=Online
-        )
+        if available_filter:
+            predicate.update(
+                dict(
+                    present=True,
+                    ready=True,
+                    using=False,
+                    owner=None,
+                    status=STATUS_ONLINE)
+            )
 
         self.logger.debug(
             f"Find devices with requirements: {json.dumps(requirements)}, using fields: {','.join(fields)}")
@@ -181,40 +182,48 @@ class StfClient(Logger):
 
         return filter_(devices, predicate)
 
+    def list_online_devices(self, requirements: dict):
+        suitable_devices = self.list_devices(requirements=requirements)
+        online = filter(lambda device: device.get('status') == STATUS_ONLINE, suitable_devices)
+        return list(online)
+
+
     def find_and_allocate(self, requirements: dict,
                           timeout_seconds: int = DEFAULT_ALLOCATION_TIMEOUT_SECONDS,
                           shuffle: bool = True) -> dict:
         """
-        Find device based on requirements and allocate first
+        Find device based on requirements and allocate first.
+        Note that this method doesn't wait for device to be free.
+
         :param requirements: dictionary about requirements, e.g. `dict(platform='android')`
         :param timeout_seconds: allocation timeout when idle, see more from allocation api.
         :param shuffle: randomize allocation
         :return: device dictionary
+
+        :raises DeviceNotFound: suitable device not found or all devices are allocated already
         """
         NotConnectedError.invariant(self._client, 'Not connected')
-        suitable_devices = self.list_devices(requirements=requirements)
+        suitable_devices = self.list_online_devices(requirements=requirements)
         DeviceNotFound.invariant(len(suitable_devices), 'no suitable devices found')
         if shuffle:
             random.shuffle(suitable_devices)
 
         self.logger.debug(f'Found {len(suitable_devices)} suitable devices, try to allocate one')
 
-        def allocate_first():
-            def try_allocate(device):
-                try:
-                    return self.allocate(device, timeout_seconds=timeout_seconds)
-                except (AssertionError, ForbiddenException) as error:
-                    self.logger.warning(f"{device.get('serial')}Allocation fails: {error}")
-                    return None
+        def try_allocate(device_candidate):
+            try:
+                return self.allocate(device_candidate, timeout_seconds=timeout_seconds)
+            except (AssertionError, ForbiddenException) as error:
+                self.logger.warning(f"{device_candidate.get('serial')} allocation fails: {error}")
+                return None
 
-            tasks = map_(suitable_devices, lambda item: wrap(item, try_allocate))
-            return find(tasks, lambda allocFunc: allocFunc())
+        # generate try_allocate tasks for suitable devices
+        tasks = map_(suitable_devices, lambda item: wrap(item, try_allocate))
+        # find first successful allocation
+        result = find(tasks, lambda allocFunc: allocFunc())
 
-        result = allocate_first()
-        if not result:
-            raise DeviceNotFound()
-        device = result.args[0]
-        return device
+        DeviceNotFound.invariant(result, 'no suitable devices found')
+        return result.args[0]
 
     def find_wait_and_allocate(self,
                                requirements: dict,
@@ -229,19 +238,30 @@ class StfClient(Logger):
         :param shuffle: allocate suitable device randomly.
         :return: device dictionary
         """
-        device = None
-        for i in range(wait_timeout):  # try to allocate for 1 minute..
+        wait_until = time.time() + wait_timeout
+
+        # Fail fast if no suitable devices
+        suitable_devices = self.list_devices(requirements=requirements, available_filter=False)
+        if not suitable_devices:
+            raise DeviceNotFound(f'No suitable devices found ({json.dumps(requirements)})')
+
+        print(f'wait_until: {wait_until}')
+        while True:
+            remaining_time = int(wait_until - time.time())
+            print(f'remaining_time: {remaining_time}')
             try:
-                device = self.find_and_allocate(requirements=requirements,
-                                                timeout_seconds=timeout_seconds,
-                                                shuffle=shuffle)
-                break
+                return self.find_and_allocate(requirements=requirements,
+                                              timeout_seconds=timeout_seconds,
+                                              shuffle=shuffle)
             except DeviceNotFound:
                 # Wait a while
-                time.sleep(1)
-                pass
-        DeviceNotFound.invariant(device, 'Suitable device not found')
-        return device
+                self.logger.debug(f'Suitable device not available, '
+                                  f'wait a while and try again. Timeout in {remaining_time} seconds')
+            if (wait_until - time.time()) <= 0:
+                break
+            # Wait a while to avoid too frequent polling
+            time.sleep(1)
+        raise DeviceNotFound(f'Suitable device not found within {wait_timeout}s timeout ({json.dumps(requirements)})')
 
     @contextmanager
     def allocation_context(self, requirements: dict,
@@ -264,7 +284,5 @@ class StfClient(Logger):
         self.logger.info(f'device allocated: {device}')
         adb_adr = self.remote_connect(device)
         device['remote_adb_url'] = adb_adr
-        self.remote_connect(device)
         yield device
         self.release(device)
-
