@@ -9,8 +9,6 @@ from stf_appium_client.StfClient import StfClient
 from stf_appium_client.exceptions import *
 
 
-
-
 class TestStfClientBasics(unittest.TestCase):
 
     @classmethod
@@ -109,6 +107,14 @@ class TestStfClient(unittest.TestCase):
         with self.assertRaises(DeviceNotFound):
             self.client.find_and_allocate({})
 
+    def test_fail_fast(self):
+
+        class MockResp:
+            devices = [{'serial': 123, 'present': True, 'ready': True, 'using': False, 'owner': None, 'status': 1}]
+        self.DevicesApi.return_value.get_devices = MagicMock(return_value=MockResp())
+        with self.assertRaises(DeviceNotFound):
+            self.client.find_and_allocate({})
+
     def test_list_devices(self):
         available = {'serial': 123, 'present': True, 'ready': True, 'using': False, 'owner': None, 'status': 3}
         self.client.get_devices = MagicMock(return_value=[available])
@@ -207,11 +213,31 @@ class TestStfClient(unittest.TestCase):
     @patch('time.sleep', side_effect=MagicMock())
     def test_allocation_context_wait_success(self, mock_sleep):
         dev1 = {'serial': '123', 'present': True, 'ready': True, 'using': False, 'owner': None, 'status': 3}
-        self.client.get_devices = MagicMock(side_effect=[[], [dev1]])
+        self.client.get_devices = MagicMock(side_effect=[[dev1], [], [dev1]])
         url = '123'
 
         with self.client.allocation_context({"serial": '123'}) as device:
             self.assertEqual(device['serial'], '123')
             self.assertEqual(device['remote_adb_url'], url)
 
+    @patch('time.sleep', side_effect=MagicMock())
+    def test_allocation_context_timeout(self, mock_sleep):
+        dev1 = {'serial': '123', 'present': True, 'ready': True, 'using': True, 'owner': "asd", 'status': 3}
+        self.client.get_devices = MagicMock(return_value=[dev1])
 
+        with self.assertRaises(DeviceNotFound) as error:
+            with self.client.allocation_context({"serial": '123'}, wait_timeout=0) as device:
+                pass
+        self.assertEqual(str(error.exception), 'Suitable device not found within 0s timeout ({"serial": "123"})')
+
+    @patch('time.sleep', return_value=MagicMock())
+    @patch('time.time')
+    def test_allocation_context_timeout_long(self, mock_time, mock_sleep):
+        dev1 = {'serial': '123', 'present': True, 'ready': True, 'using': True, 'owner': "asd", 'status': 3}
+        self.client.get_devices = MagicMock(return_value=[dev1])
+        self.client.stf_find_and_allocate = MagicMock(side_effect=DeviceNotFound)
+        mock_time.side_effect = [0, 0, 0, 10, 10, 10]
+        with self.assertRaises(DeviceNotFound) as error:
+            with self.client.allocation_context({"serial": '123'}, wait_timeout=10):
+                pass
+        self.assertEqual(str(error.exception), 'Suitable device not found within 10s timeout ({"serial": "123"})')
